package com.ebuddy.hornetq.test;

import org.hornetq.api.core.HornetQException;
import org.hornetq.api.core.client.*;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @author Joost van de Wijgerd <jvandewijgerd@ebuddy.com>
 */
public class TestMessageHandler {
    public static void main(String... args) {
        // load the app context
        ApplicationContext appCtx = new ClassPathXmlApplicationContext("eventservice-beans.xml");
        ClientSessionFactory sessionFactory = (ClientSessionFactory) appCtx.getBean("clientSessionFactory");
        final CountDownLatch waitLatch = new CountDownLatch(1000000);
        try {
            EventPluginRunner runner = new EventPluginRunner(sessionFactory.createSession(), waitLatch);
            runner.start();
            waitLatch.await();
            runner.destroy();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private static final class EventPluginRunner implements MessageHandler {
        private final ClientSession session;
        private ClientConsumer consumer;
        private final CountDownLatch waitLatch;
        private final AtomicLong counter = new AtomicLong(0);


        private EventPluginRunner(ClientSession session, CountDownLatch waitLatch) {
            this.session = session;
            this.waitLatch = waitLatch;
        }

        @Override
        public void onMessage(ClientMessage clientMessage) {
            try {
                long count = counter.incrementAndGet();
                if(count % 10000 == 0) {
                    System.out.println("got "+count+" messages");
                }
                waitLatch.countDown();
            } finally {
                try {
                    clientMessage.acknowledge();
                } catch (HornetQException e) {
                    // ignore
                }
            }
        }

        public void start() throws HornetQException {
            consumer = session.createConsumer("eBuddy.events.SECURITY.login.success");
            consumer.setMessageHandler(this);
            session.start();
        }

        public void destroy() throws HornetQException {
            consumer.close();
            session.close();
        }
    }
}
