package com.ebuddy.hornetq.test;

import org.hornetq.api.core.HornetQException;
import org.hornetq.api.core.client.ClientMessage;
import org.hornetq.api.core.client.ClientProducer;
import org.hornetq.api.core.client.ClientSession;
import org.hornetq.api.core.client.ClientSessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.testng.AbstractTestNGSpringContextTests;
import org.testng.annotations.Test;

import java.io.IOException;
import java.util.concurrent.*;

/**
 * @author Joost van de Wijgerd <jvandewijgerd@ebuddy.com>
 */
@ContextConfiguration(locations = {"classpath:test-resources.xml"})
public class EventPluginTest extends AbstractTestNGSpringContextTests {
    private ClientSessionFactory sessionFactory;

    @Autowired
    public void setSessionFactory(ClientSessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }


    @Test(enabled = false)
    public void loadTestSecurityEventsWithFilters() throws IOException, ExecutionException, HornetQException, InterruptedException {
        loadTestSecurityEvents("eBuddy.events");
    }

    @Test(enabled = false)
    public void loadTestSecurityEventsWithoutFilters() throws IOException, ExecutionException, HornetQException, InterruptedException {
        loadTestSecurityEvents("eBuddy.events.SECURITY.login.success");
    }


    private void loadTestSecurityEvents(String queueName) throws IOException, HornetQException, ExecutionException, InterruptedException {
        //ClientSession session = sessionFactory.createSession();
        //session.start();

        // data is in the following format
        // session_hash,      [0]
        // email,                 [1]
        // remote_ip,          [2]
        // login_type,          [3]
        // country,              [4]
        // network ,            [5]
        // version_type,       [6]
        // frontend_version, [7]
        // server_id,           [8]
        // login_time,          [9]
        // duration ,           [10]
        // loginsuccess,      [11]
        // useragent,         [12]
        // msg_sent,         [13]
        // msg_rcv            [14]
        int numProducers = 5;

        final ExecutorService executor = Executors.newFixedThreadPool(numProducers + 1);
        final BlockingQueue<String> lineQueue = new LinkedBlockingQueue<String>();
        final CountDownLatch waitLatch = new CountDownLatch(numProducers);

        // first add the ReadDataTask
        final ReadDataTask dataReader = new ReadDataTask(lineQueue);
        Future<Long> dataReaderFuture = executor.submit(dataReader);

        // wait some seconds for the reader to warm up
        Thread.sleep(5000);

        long start = System.currentTimeMillis();
        for (int i = 0; i < numProducers; i++) {
            executor.submit(new ProducerTask(sessionFactory.createSession(), queueName, lineQueue, waitLatch));
        }

        waitLatch.await();

        long duration = System.currentTimeMillis() - start;

        System.out.println("send " + dataReaderFuture.get() + " messages in " + duration + " msecs");

        //session.close();
    }

    private static final class ReadDataTask implements Callable<Long> {
        private final BlockingQueue<String> lineQueue;

        private ReadDataTask(BlockingQueue<String> lineQueue) {
            this.lineQueue = lineQueue;
        }

        @Override
        public Long call() throws Exception {
            /*Resource r = new ClassPathResource("/testdata.csv.gz");
            GZIPInputStream in = new GZIPInputStream(r.getInputStream());
            BufferedReader lineReader = new BufferedReader(new InputStreamReader(in, Charset.forName("UTF-8")));
            String line = null;*/
            long count = 0;
            while (count < 500000) {
                lineQueue.offer("31937012c49fc02b793632608e82611c\txxxxxx@hotmail.com\t188.160.82.221\tMSN_J2ME2\tSY\tMSN\tJ2ME2\t1.4.2\t416\t23:59:59\t124\t1\tnokia_series60v2-176x208 / 1.4.2 / getjar\t1\t3");
                lineQueue.offer("fbac3912b8ce4983b1436e09f6706ddb\txxxx@hotmail.com\t212.66.77.38\tMSN_J2ME2\tLU\tMSN\tJ2ME2\t1.4.2\t327\t23:59:59\t52\t1\tse_w910 / 1.4.2 / ebuddy\t0\t0");
                lineQueue.offer("26aea1a1dbdf537af7c0e796c4ca3744\txxxxx@hotmail.it\t217.201.67.69\tMSN_J2ME2\tIT\tMSN\tJ2ME2\t1.4.1\t453\t23:59:59\t77\t1\tnokia_n73 / 1.4.1 / ebuddy\t0\t0");
                lineQueue.offer("fa60f1c8d96f372b2fe64d2bc7d8f823\txxxxxx\t213.178.224.178\tMASTER_J2ME2\tSY\tMASTER\tJ2ME2\t1.5.0\t406\t23:59:59\t490\t1\tnokia_series40v3-240x320\t0\t0");
                lineQueue.offer("694d85c7718ecd635494b5179f764fdc\txxxxxxxxxxxx@hotmail.com\t188.160.126.226\tMSN_J2ME2\tSY\tMSN\tJ2ME2\t1.4.2\t441\t23:59:59\t0\t1\tse_240x320 / 1.4.2 / getjar\t0\t0");
                count += 5;
            }
            //lineReader.close();
            return count;
        }
    }

    private static final class ProducerTask implements Callable<Long> {
        private final BlockingQueue<String> lineQueue;
        private final ClientSession session;
        private final CountDownLatch waitLatch;
        private final String queueName;

        private ProducerTask(ClientSession session, String queueName, BlockingQueue<String> lineQueue, CountDownLatch waitLatch) {
            this.session = session;
            this.lineQueue = lineQueue;
            this.waitLatch = waitLatch;
            this.queueName = queueName;
        }

        @Override
        public Long call() throws Exception {
            try {
                String line = null;
                ClientProducer successProducer = session.createProducer(queueName);

                long count = 0;
                while ((line = lineQueue.poll(2, TimeUnit.SECONDS)) != null) {
                    String[] positions = line.split("\t");
                    StringBuilder builder = new StringBuilder(256);
                    builder.append("hash=").append(positions[0]).append(";");
                    builder.append("account=").append(positions[1]).append(";");
                    builder.append("country=").append(positions[4]).append(";");
                    builder.append("network=").append(positions[5]).append(";");
                    builder.append("client=").append(positions[6]).append(";");
                    builder.append("client_version=").append(positions[7]).append(";");
                    builder.append("login_time=").append(positions[9]).append(";");
                    builder.append("ua=").append(positions[12]).append(";");
                    // ok we have the body now get the properties
                    long ip = 0;
                    try {
                        ip = ipToLong(positions[2]);
                    } catch (NumberFormatException e) {
                        continue;
                    }
                    boolean success = "1".equals(positions[11]);
                    ClientMessage message = session.createMessage(false);
                    message.putStringProperty("type", "SECURITY");
                    message.putLongProperty("ip", ip);
                    message.putBooleanProperty("success", success);
                    message.getBodyBuffer().writeString(builder.toString());
                    if (success) successProducer.send(message);
                    count += 1;
                }
                successProducer.close();
                session.close();
                return count;
            } catch (Exception e) {
                e.printStackTrace();
                throw e;
            } finally {
                waitLatch.countDown();
            }

        }
    }

    public static long ipToLong(String ip) {
        String[] octets = ip.split("\\.");
        return (Long.parseLong(octets[0]) << 24) + (Integer.parseInt(octets[1]) << 16) +
                (Integer.parseInt(octets[2]) << 8) + Integer.parseInt(octets[3]);
    }

}